<?php
/**
 * SkeletonCart
 * (c) 2017 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace MTDBConnector;

use ORM;

class Plugin extends \App\Plugin
{
    const OPTION_COLUMN_OFFSET = 5;
    const BLOG_ID_COLUMN = 11;

    protected $fileName;
    protected $config;
    protected $mtDBConfig;
    protected $mtPluginSetting;

    public function __construct($key)
    {
        parent::__construct($key);
        $this->fileName = __DIR__ . '/Config.php';
        $this->mtPluginSetting = array();

        $this->loadConfig();
        $this->loadMTConfig();
        $this->initDatabase();
        $this->initCallback();
    }

    public function loadConfig()
    {
        if (!is_readable($this->fileName)) {
            throw new \RuntimeException("MTDBConnector設定が読み込めません。");
        }
        $this->config = include $this->fileName;

        if (!isset($this->config['mtConfigFile']) || !is_readable($this->config['mtConfigFile'])) {
            throw new \RuntimeException("MTConfigファイルにアクセスできません。");
        }
        if (!isset($this->config['useEntryID']) || !is_numeric($this->config['useEntryID'])) {
            throw new \RuntimeException("useEntryIDが設定されていません。");
        }
        if (!isset($this->config['mtBaseDir']) || !is_readable($this->config['mtBaseDir'])) {
            throw new \RuntimeException("mtBaseDirが設定されていないかアクセスできません。");
        }
        require_once $this->config['mtBaseDir'] . '/php/lib/MTSerialize.php';
    }

    public function loadMTConfig()
    {
        $contents = file_get_contents($this->config['mtConfigFile']);
        if ($contents === false) {
            throw new \RuntimeException("MTConfigファイルを読み込めません。");
        }
        if (strlen($contents) == 0) {
            throw new \RuntimeException("MTConfigファイルが空です。");
        }
        $mtDBConfig = array(
            'Database' => null,
            'DBUser' => null,
            'DBPassword' => null,
            'DBHost' => null,
        );
        $lines = explode("\n", $contents);
        foreach ($lines as $line) {
            if (preg_match('/^\s*(\w+)\s+([^\s]+)\s*$/', $line, $matches)) {
                $key = $matches[1];
                $value = $matches[2];
                if (array_key_exists($key, $mtDBConfig) !== false) {
                    $mtDBConfig[$key] = $value;
                }
            }
        }
        array_walk($mtDBConfig, function ($item, $key) {
            if (is_null($item) || strlen($item) === 0) {
                throw new \RuntimeException("MTConfigファイルに{$key}が設定されていません。");
            }
        });
        $this->mtDBConfig = $mtDBConfig;
    }

    public function initDatabase()
    {
        ORM::configure(array(
            'connection_string'
                => "mysql:host={$this->mtDBConfig['DBHost']};dbname={$this->mtDBConfig['Database']}",
            'username' => $this->mtDBConfig['DBUser'],
            'password' => $this->mtDBConfig['DBPassword'],
        ));
        ORM::configure('id_column_overrides', array(
            'mt_sc_item' => 'sc_item_id',
            'mt_plugindata' => 'plugindata_id',
        ));
        ORM::configure('driver_options',
            array(\PDO::MYSQL_ATTR_INIT_COMMAND => 'SET NAMES utf8')
        );
    }

    public function initCallback()
    {
        \App::addCallback('postInit', array($this, 'postInit'));
        \App::addCallback('updateCartItem', array($this, 'updateCartItem'));
    }

    public function postInit()
    {
        $mtDBConfig = $this->mtDBConfig;
        $useEntryID = $this->config['useEntryID'];
        \App::register('itemList', '\MTDBConnector\MTDBItemList', array($mtDBConfig, $useEntryID));
        \App::register('stockAllocator', '\MTDBConnector\StockAllocator');
    }

    protected function getPluginSetting($blogId)
    {
        if(!array_key_exists($blogId, $this->mtPluginSetting)) {
            $pdata = ORM::for_table('mt_plugindata')
                        ->where('plugindata_key', 'configuration:blog:'.$blogId)
                        ->where('plugindata_plugin', 'SkeletonCartConnector')
                        ->find_one();
            if($pdata === false) {
                return false;
            }
            $data = $pdata->get('plugindata_data');
            $ser = new \MTSerialize();
            $setting = $ser->unserialize($data);
            $this->mtPluginSetting[$blogId] = $setting;
        }
        return $this->mtPluginSetting[$blogId];
    }

    protected function getOptionFieldId($blogId, $optionId)
    {
        $setting = $this->getPluginSetting($blogId);
        if(is_null($setting)) {
            return null;
        }
        foreach(range(1, 5) as $n) {
            if(isset($setting['item_option_number_'.$n])) {
                $id = $setting['item_option_number_'.$n];
                if($id >= 1 && $id <= 30 && $id == $optionId) {
                    return $n;
                }
            }
        }
        return null;
    }

    protected function getPriceListForOption($id, $optionId)
    {
        if ($optionId > 30 || $optionId <= 0) {
            throw new \RuntimeException('オプション番号は1から30までサポートされます。');
        }
        $itemList = \App::itemList();
        $rawItem = $itemList->findOne($id);
        $blogId = $rawItem[self::BLOG_ID_COLUMN];
        $optionFieldId = $this->getOptionFieldId($blogId, $optionId);
        if(is_null($optionFieldId)) {
            return null;
        }
        $col = $optionFieldId + self::OPTION_COLUMN_OFFSET;
        if (!array_key_exists($col, $rawItem)) {
            return null;
        }
        $optionStr = $rawItem[$col];
        $lines = explode("\n", $optionStr);
        $lines = array_map('trim', $lines);
        $lines = array_filter($lines, 'strlen');
        $lines = array_values($lines);
        $options = array();
        foreach ($lines as $line) {
            $pair = explode(',', $line);
            $options[trim($pair[0])] = count($pair) == 2 ? trim($pair[1]) : 0;
        }
        return $options;
    }

    public function updateCartItem($cartItem)
    {
        $totalOptionsPrice = 0;
        foreach ($cartItem->options() as $n => $values) {
            $priceList = $this->getPriceListForOption($cartItem->id(), $n);
            if (is_null($priceList)) {
                continue;
            }
            if (!is_array($values) || !array_key_exists('value', $values)) {
                continue;
            }
            $optionValue = $values['value'];
            if (array_key_exists($optionValue, $priceList)) {
                $values['price'] = $priceList[$optionValue];
                $totalOptionsPrice += $priceList[$optionValue];
                $cartItem->setOption($n, $values);
            }
        }
        $cartItem->setPrice($cartItem->unitPrice() + $totalOptionsPrice);
        $cartItem->setTotalPrice($cartItem->price() * $cartItem->amount());
    }
}

\App::addPlugin(new Plugin('MTDBConnector'));

